@@ -18,7 +18,6 @@
 #include "soup-message-queue.h"
 #include "soup-misc.h"
 #include "soup-socket.h"
-#include "soup-ssl.h"
 
 typedef enum {
 	SOUP_MESSAGE_IO_CLIENT,
@@ -339,6 +338,7 @@ content_decode_one (SoupBuffer *buf, GConverter *converter, GError **error)
 	gsize outbuf_length, outbuf_used, outbuf_cur, input_used, input_cur;
 	char *outbuf;
 	GConverterResult result;
+	gboolean dummy_zlib_header_used = FALSE;
 
 	outbuf_length = MAX (buf->length * 2, 1024);
 	outbuf = g_malloc (outbuf_length);
@@ -358,6 +358,39 @@ content_decode_one (SoupBuffer *buf, GConverter *converter, GError **error)
 			g_clear_error (error);
 			outbuf_length *= 2;
 			outbuf = g_realloc (outbuf, outbuf_length);
+		} else if (input_cur == 0 &&
+			   !dummy_zlib_header_used &&
+			   G_IS_ZLIB_DECOMPRESSOR (converter) &&
+			   g_error_matches (*error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA)) {
+
+			GZlibCompressorFormat format;
+			g_object_get (G_OBJECT (converter), "format", &format, NULL);
+
+			if (format == G_ZLIB_COMPRESSOR_FORMAT_ZLIB) {
+				/* Some servers (especially Apache with mod_deflate)
+				 * return RAW compressed data without the zlib headers
+				 * when the client claims to support deflate. For
+				 * those cases use a dummy header (stolen from
+				 * Mozilla's nsHTTPCompressConv.cpp) and try to
+				 * continue uncompressing data.
+				 */
+				static char dummy_zlib_header[2] = { 0x78, 0x9C };
+
+				g_converter_reset (converter);
+				result = g_converter_convert (converter,
+							      dummy_zlib_header, sizeof(dummy_zlib_header),
+							      outbuf + outbuf_cur, outbuf_length - outbuf_cur,
+							      0, &input_used, &outbuf_used, NULL);
+				dummy_zlib_header_used = TRUE;
+				if (result == G_CONVERTER_CONVERTED) {
+					g_clear_error (error);
+					continue;
+				}
+			}
+
+			g_free (outbuf);
+			return NULL;
+
 		} else if (*error) {
 			/* GZlibDecompressor can't ever return
 			 * G_IO_ERROR_PARTIAL_INPUT unless we pass it
@@ -1077,27 +1110,6 @@ new_iostate (SoupMessage *msg, SoupSocket *sock, SoupMessageIOMode mode,
 	io->read_state  = SOUP_MESSAGE_IO_STATE_NOT_STARTED;
 	io->write_state = SOUP_MESSAGE_IO_STATE_NOT_STARTED;
 
-	if (soup_socket_is_ssl (io->sock)) {
-		GTlsCertificate *certificate;
-		GTlsCertificateFlags errors;
-
-		g_object_get (sock,
-			      SOUP_SOCKET_TLS_CERTIFICATE, &certificate,
-			      SOUP_SOCKET_TLS_ERRORS, &errors,
-			      NULL);
-		g_object_set (msg,
-			      SOUP_MESSAGE_TLS_CERTIFICATE, certificate,
-			      SOUP_MESSAGE_TLS_ERRORS, errors,
-			      NULL);
-		if (certificate)
-			g_object_unref (certificate);
-	} else {
-		g_object_set (msg,
-			      SOUP_MESSAGE_TLS_CERTIFICATE, NULL,
-			      SOUP_MESSAGE_TLS_ERRORS, 0,
-			      NULL);
-	}
-
 	if (priv->io_data)
 		soup_message_io_cleanup (msg);
 	priv->io_data = io;
